<?php

namespace Quriiri_API;

class Sender
{
    const VERSION = "1.0.1";

    private $curl;

    public function __construct($apikey, $url)
    {
        $this->curl = curl_init($url);
        curl_setopt_array($this->curl, array(
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_USERAGENT => self::user_agent(),
            CURLOPT_HTTPHEADER => array(
                "Accept:",
                "Authorization: apikey $apikey",
                "Content-Type: application/json",
            ),
            CURLOPT_RETURNTRANSFER => TRUE,
        ));
    }

    public function __destruct()
    {
        curl_close($this->curl);
    }

    private static function user_agent()
    {
        return sprintf("Quriiri-TX-PHP/%s curl/%s PHP/%s",
                       self::VERSION, curl_version()["version"], phpversion());
    }

    private static function fill_messages(&$ret, $destinations, $status)
    {
        if (!array_key_exists("messages", $ret))
            $ret["messages"] = array();
        foreach($destinations as $destination) {
            if (!array_key_exists($destination, $ret["messages"])) {
                $ret["messages"][$destination] = array(
                    "converted" => $destination, "status" => $status);
            }
        }
    }

    /**
     * Send SMS to a number of destinations.
     * All strings in parameters must be UTF-8 encoded.
     *
     * @param string $sender Sender
     * @param mixed  $destination Destination; string or array of strings
     * @param string $text Text to send
     * @param array  $optional optional parameters as specified in the API spec
     *        + verbose_fh = filehandle opened for writing verbose HTTP logging
     *
     * @return associative array with errors, warnings, and messages keys
     */
    public function sendSms($sender, $destination, $text, $optional = array())
    {
        $data = array_merge(array(
            "sender" => $sender,
            "destination" => $destination,
            "text" => $text,
            "flash" => FALSE,
        ), $optional);

        if ($data["flash"])
            $data["flash"] = "true";
        else
            unset($data["flash"]);
        foreach($data as $key => $value) {
            if ($key == "destination" || strpos($key, "cr_", 0) === 0) {
                if (!is_array($value))
                    $data[$key] = array($value);
            }
        }

        $verbose_fh = isset($data["verbose_fh"]) ? $data["verbose_fh"] : NULL;
        unset($data["verbose_fh"]);
        if ($verbose_fh) {
            curl_setopt($this->curl, CURLOPT_VERBOSE, TRUE);
            curl_setopt($this->curl, CURLOPT_STDERR, $verbose_fh);
        }
        else {
            curl_setopt($this->curl, CURLOPT_VERBOSE, FALSE);
            curl_setopt($this->curl, CURLOPT_STDERR, NULL);
        }

        curl_setopt($this->curl, CURLOPT_POSTFIELDS, json_encode($data));

        $res = curl_exec($this->curl);

        $implicit_status = "UNKNOWN";
        if ($res === FALSE) {
            # http://curl.haxx.se/libcurl/c/libcurl-errors.html
            # TODO: 28 = CURLE_OPERATION_TIMEDOUT too?
            # 56 = CURLE_RECV_ERROR
            if (curl_errno($this->curl) == 56) {
                $key = "warnings";
            }
            else {
                $key = "errors";
                $implicit_status = "FAILED";
            }
            $res = array($key => array(
                array("message" => curl_error($this->curl))));
        }
        else {
            $res = json_decode($res, TRUE);
            if (!$res) {
                $info = curl_getinfo($this->curl);
                if ($info["http_code"] >= 400) {
                    $key = "errors";
                    $message = "HTTP " . $info["http_code"];
                    $implicit_status = "FAILED";
                }
                else {
                    $key = "warnings";
                    $message = "JSON decode error " . json_last_error();
                }
                $res = array($key => array(array("message" => $msg)));
            }
            else if (array_key_exists("errors", $res) && $res["errors"]) {
                $implicit_status = "FAILED";
            }
        }
        self::fill_messages($res, $destination, $implicit_status);

        if (!array_key_exists("errors", $res))
            $res["errors"] = array();
        if (!array_key_exists("warnings", $res))
            $res["warnings"] = array();

        return $res;
    }
}

?>
